#include "str9100/porting.h"
#include "str9100/flib_flash.h"


void    Do_Delay(unsigned count);

void    Check_Toggle_Ready(unsigned char *Dst);
void    Check_Toggle_ReadyHalfWord(unsigned short *Dst);
void    Check_Toggle_ReadyWord(unsigned *Dst);


void fLib_Flash_ReadID(unsigned flash_type, unsigned base, unsigned BusWidth, unsigned* DDI1, unsigned* DDI2)
{
    if (flash_type == MX_Type)
    {
        switch(BusWidth)
        {
            case FLASH_SINGLE: // 8 bit
                cpe_outl(base + 0x555, 0xaa);
	        cpe_outl(base + 0x2aa, 0x55);
	        cpe_outl(base + 0x555, 0x90);
	        
	        Check_Toggle_Ready(base + 0x00);
	       
	        *DDI1 = cpe_inl(base + 0x00);
	    
	        cpe_outl(base + 0x555, 0xaa);
	        cpe_outl(base + 0x2aa, 0x55);
	        cpe_outl(base + 0x555, 0x90);
	        
	        Check_Toggle_Ready(base + 0x01);
	        
	        *DDI2 = cpe_inl(base + 0x01);
	        
	        break;
	        
	    case FLASH_DOUBLE: // double 16 bit
	        cpe_outl(base+0x555*2,0xaaaa);
	        cpe_outl(base+0x2aa*2,0x5555);
	        cpe_outl(base+0x555*2,0x9090);
	        Check_Toggle_ReadyHalfWord(base+0x00);
	        *DDI1 = cpe_inl(base+0x00);
	    
	        cpe_outl(base+0x555*2,0xaaaa);
	        cpe_outl(base+0x2aa*2,0x5555);
	        cpe_outl(base+0x555*2,0x9090);
	        Check_Toggle_ReadyHalfWord(base+0x02);
	        *DDI2 = cpe_inl(base+0x02);
	        break;
	        
	    case FLASH_FOUR:// four 32 bit
	        cpe_outl(base+0x555*4,0xaaaaaaaa);
	        cpe_outl(base+0x2aa*4,0x55555555);
	        cpe_outl(base+0x555*4,0x90909090);
	        Check_Toggle_ReadyWord(base+0x00);
	        *DDI1 = cpe_inl(base+0x00);
	    
	        cpe_outl(base+0x555*4,0xaaaaaaaa);
	        cpe_outl(base+0x2aa*4,0x55555555);
	        cpe_outl(base+0x555*4,0x90909090);
	        Check_Toggle_ReadyWord(base+0x04);
	        *DDI2 = cpe_inl(base+0x04);
	        break;
        }
    }
}


void fLib_Flash_Reset(unsigned base, unsigned BusWidth)
{
    switch(BusWidth)
    {
        case FLASH_SINGLE:
            cpe_outl(base,0xf0);
            break;
            
        case FLASH_DOUBLE:
            cpe_outl(base,0xf0f0);
            break;
            
        case FLASH_FOUR:
            //cpe_outl(base,0xf0f0f0f0);
            cpe_outl(base,0x00ff00ff); /* modify by Charles Tsai */
        break;
    }
}


void fLib_Flash_ChipErase(unsigned flash_type, unsigned base, unsigned BusWidth)
{
    unsigned data;

    if (flash_type == MX_Type)
    {
        switch(BusWidth)
        {
	    case FLASH_SINGLE: // 8 bit
	        cpe_outl(base+0x555,0xaa);
	        cpe_outl(base+0x2aa,0x55);
	        cpe_outl(base+0x555,0x80);
	        cpe_outl(base+0x555,0xaa);        
	        cpe_outl(base+0x2aa,0x55);            
	        cpe_outl(base+0x555,0x10);
	        break;
	        
	    case FLASH_DOUBLE: // double 16 bit
	        cpe_outl(base+0x555*2,0xaaaa);
	        cpe_outl(base+0x2aa*2,0x5555);
	        cpe_outl(base+0x555*2,0x8080);
	        cpe_outl(base+0x555*2,0xaaaa);        
	        cpe_outl(base+0x2aa*2,0x5555);            
	        cpe_outl(base+0x555*2,0x1010);
	        break;
	        
	    case FLASH_FOUR:// four 32 bit
	        cpe_outl(base+0x555*4,0xaaaaaaaa);
	        cpe_outl(base+0x2aa*4,0x55555555);
	        cpe_outl(base+0x555*4,0x80808080);
	        cpe_outl(base+0x555*4,0xaaaaaaaa);        
	        cpe_outl(base+0x2aa*4,0x55555555);            
	        cpe_outl(base+0x555*4,0x10101010);
	        break;
        }
    }
}


void fLib_Flash_SectorErase(unsigned flash_type, unsigned base, unsigned BusWidth, unsigned sector)
{
    unsigned data;

    if (flash_type == MX_Type)
    {
        switch(BusWidth)
        {
	    case FLASH_SINGLE: // 8 bit
	        cpe_outl(base+0x555,0xaa);
	        cpe_outl(base+0x2aa,0x55);
	        cpe_outl(base+0x555,0x80);
	        cpe_outl(base+0x555,0xaa);        
	        cpe_outl(base+0x2aa,0x55);            
	        cpe_outl(base+sector,0x30);                
	        break;
	        
	    case FLASH_DOUBLE: // double 16 bit
	        cpe_outl(base+0x555*2,0xaaaa);
	        cpe_outl(base+0x2aa*2,0x5555);
	        cpe_outl(base+0x555*2,0x8080);
	        cpe_outl(base+0x555*2,0xaaaa);        
	        cpe_outl(base+0x2aa*2,0x5555);            
	        cpe_outl(base+sector*2,0x3030);                
	        break;
	        
	    case FLASH_FOUR:// four 32 bit
	        cpe_outl(base+0x555*4,0xaaaaaaaa);
	        cpe_outl(base+0x2aa*4,0x55555555);
	        cpe_outl(base+0x555*4,0x80808080);
	        cpe_outl(base+0x555*4,0xaaaaaaaa);        
	        cpe_outl(base+0x2aa*4,0x55555555);            
	        cpe_outl(base+sector*4,0x30303030);                
	        break;
	    }
    }
}


/************************************************************************/
/* PROCEDURE:    Check_Toggle_Ready                                     */
/*                                                                      */
/* During the internal program cycle, any consecutive read operation    */
/* on DQ6 will produce alternating 0's and 1's (i.e. toggling between   */
/* 0 and 1). When the program cycle is completed, DQ6 of the data will  */
/* stop toggling. After the DQ6 data bit stops toggling, the device is  */
/* ready for next operation.                                            */
/*                                                                      */
/* Input:                                                               */
/*           Dst        must already be set-up by the caller            */
/*                                                                      */
/* Output:                                                              */
/*           None                                                       */
/************************************************************************/

void Check_Toggle_Ready (unsigned char *Dst)
{
    unsigned char Loop = 1;
    unsigned char PreData;
    unsigned char CurrData;
    long long TimeOut = 0;

    PreData = *Dst;
    PreData = PreData & 0x40;
    
    while ((TimeOut< 0x07FFFFFF) && (Loop))
    {
        CurrData = *Dst;
        
        CurrData = CurrData & 0x40;
            
        if (PreData == CurrData)
        {
            Loop = 0;   /* ready to exit the while loop */
        }
                    
        PreData = CurrData;
        
        TimeOut++;
    }
}


void Check_Toggle_ReadyHalfWord(unsigned short *Dst)
{
    unsigned char Loop = 1;
    short PreData;
    short CurrData;
    long long TimeOut = 0;


    PreData = *Dst;
    PreData = PreData & 0x4040;
        
    while ((TimeOut< 0x07FFFFFF) && (Loop))
    {
        CurrData = *Dst;
        CurrData = CurrData & 0x4040;
        
        if (PreData == CurrData)
        {
            Loop = 0;   /* ready to exit the while loop */
        }
                    
        PreData = CurrData;
            
        TimeOut++;
    }
}

// this function only use in FLASH_FOUR 
void fLib_Flash_ProgramWord(unsigned flash_type, unsigned base, unsigned address, unsigned data)
{
    volatile unsigned int s_reg = 0;

    if (flash_type == MX_Type) 
    {   
        cpe_outl(base+0x555*4,0xaaaaaaaa);
	cpe_outl(base+0x2aa*4,0x55555555);
	cpe_outl(base+0x555*4,0xa0a0a0a0);
	cpe_outl(address,data);
	    
	Check_Toggle_ReadyWord((unsigned  *) (address));
    }
}


void Check_Toggle_ReadyWord(unsigned  *Dst)
{
    unsigned char Loop = 2;
    unsigned PreData;
    unsigned CurrData;
    long long TimeOut = 0;

        
    PreData = *Dst;
    
    PreData = PreData & 0x40404040;
        
    while (Loop)
    {
        CurrData = *Dst;
        CurrData = CurrData & 0x40404040;
        
        if (PreData == CurrData)
        {
            Loop--;   /* ready to exit the while loop */
        }

        PreData = CurrData;
        TimeOut++;
    }
}


int Check_Toggle_ReadyWord1(unsigned base, unsigned programmed_Data, unsigned  *Dst)
{
    unsigned loopCounter;
    unsigned TimeOutFlag;
    unsigned ProgramOkFlag;
    unsigned read_Data;
    unsigned return_status;    
    unsigned i;
    
    loopCounter = 0;
    TimeOutFlag = 0;
    ProgramOkFlag = 1;
    
    while (1)
    {
        read_Data = *Dst;
        
        if (read_Data == programmed_Data)
        {
            return_status = PROGRAM_OK;
            
            break;
    	}

        /*
         * delay
         */
	for (i = 0; i < FLASH_PROGRAM_DELAY_CYCLE; i++);
	    
	if (loopCounter >= FLASH_PROGRAM_LOOP_COUNTER)
	{
	    return_status = PROGRAM_NOT_OK;
	    
            break;
	}
	    
        loopCounter++;
    }
    
    return return_status;
}


void fLib_Flash_ReadWord(unsigned address, unsigned* data)
{
    *data = cpe_inl(address);
}


void fLib_Flash_ReadHalfWord(unsigned address, unsigned short* data)
{
    *data = cpe_inl(address);
}


void fLib_Flash_ReadByte(unsigned address, unsigned char* data)
{
    *data = cpe_inl(address);
}


void Do_Delay(unsigned count)
{   
    unsigned i;
    
    for ( i = count; i>0; i--);
}