/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include "str9100_cfg.h"

#include <asm/arch/porting.h>
#include <asm/arch/star_flash.h>
#include <asm/arch/star_smc.h>


ulong myflush(void);



/* puzzle magic for lart
 * data_*_flash are def'd in flashasm.S
 */

extern u32	data_from_flash(u32);
extern u32	data_to_flash(u32);


extern void	Flash_Read_Silicon_ID(u32 *, u32 *);
extern u32	Flash_Sectors_Erase(u32, u32);
extern u32	Flash_Word_Write(u32, u16);
extern void	Flash_Word_Read(u_int32, u_int16 *);


#define PUZZLE_FROM_FLASH(x)		(x)
#define PUZZLE_TO_FLASH(x)		(x)


#define CMD_IDENTIFY			0x90909090
#define CMD_ERASE_SETUP			0x20202020
#define CMD_ERASE_CONFIRM		0xD0D0D0D0
#define CMD_PROGRAM			0x40404040
#define CMD_RESUME			0xD0D0D0D0
#define CMD_SUSPEND			0xB0B0B0B0
#define CMD_STATUS_READ			0x70707070
#define CMD_STATUS_RESET		0x50505050

#define BIT_BUSY			0x80808080
#define BIT_ERASE_SUSPEND		0x40404040
#define BIT_ERASE_ERROR			0x20202020
#define BIT_PROGRAM_ERROR		0x10101010
#define BIT_VPP_RANGE_ERROR		0x08080808
#define BIT_PROGRAM_SUSPEND		0x04040404
#define BIT_PROTECT_ERROR		0x02020202
#define BIT_UNDEFINED			0x01010101

#define BIT_SEQUENCE_ERROR      	0x30303030
#define BIT_TIMEOUT             	0x80000000


#define SRAM_BANK_SIZE_32K		0xb0
#define SRAM_BANK_SIZE_64K		0xc0
#define SRAM_BANK_SIZE_128K		0xd0
#define SRAM_BANK_SIZE_256K		0xe0
#define SRAM_BANK_SIZE_512K		0xf0
#define SRAM_BANK_SIZE_1M		0x00
#define SRAM_BANK_SIZE_2M		0x10
#define SRAM_BANK_SIZE_4M		0x20
#define SRAM_BANK_SIZE_8M		0x30
#define SRAM_BANK_SIZE_16M		0x40
#define SRAM_BANK_SIZE_32M		0x50


#define SRAM_BUS_WIDTH_8		0
#define SRAM_BUS_WIDTH_16		1
#define SRAM_BUS_WIDTH_32		2



/*
 * please change the timing parameter for different SRAM type
 */
#define TIMING_PARAMETER		(SMC_BANK_TM_RBE_1 | \
			   	 	 SMC_BANK_TM_AST_3 | \
                           	 	 SMC_BANK_TM_CTW_3 | \
                           	 	 SMC_BANK_TM_AT1_F | \
                           	 	 SMC_BANK_TM_AT2_3 | \
                           	 	 SMC_BANK_TM_WTC_3 | \
 	                           	 SMC_BANK_TM_AHT_3 | \
        	                   	 SMC_BANK_TM_TRNA_F)


flash_info_t	flash_info[CFG_MAX_FLASH_BANKS];

extern u32	SA_Base_Addr_Table[];


/*
 * Configure memory bank
 */
void sram_config(int bank_num, u32 bank_base, int write_protect, int bank_type, int bank_size, int bus_width)
{
    u32    config_reg_val;


    config_reg_val = (0x1UL << 28) | 
                     (bank_base & 0x0FFF8000) | 
                     ((write_protect & 0x01) << 11) | 
                     ((bank_type & 0x7) << 8) | 
                     (bank_size) | 
                     (bus_width);
	
    str9100_outl(0x30000000 + bank_num * 8, config_reg_val);
}


void sram_disable(int bank_num)
{
    u32    config_reg_val;
    
    
    config_reg_val = str9100_inl(0x30000000 + bank_num * 8);
    
    config_reg_val &= ~(0x1UL << 28);
    
    str9100_outl(0x30000000 + bank_num * 8, config_reg_val);
}


void sram_set_timing(int bank_num, unsigned long timing)
{
    str9100_outl(0x30000000 + bank_num * 8 + 4, timing);
}


void mxic_mx29lv640t_flash_init(bd_t *bd)
{
    int j;
	
    flash_info[0].flash_id = (MXIC_MANUFACTURE_ID & FLASH_VENDMASK) | 
                             (MXIC_DEVICE_ID_MX29LV640T & FLASH_TYPEMASK);
    
    flash_info[0].size = 0x800000;
    
    flash_info[0].sector_count = 134 + 1;
    
    memset(flash_info[0].protect, 0, 134 + 1);
   
    for (j = 0; j < flash_info[0].sector_count; ++j)
    {
        flash_info[0].start[j] = SA_Base_Addr_Table[j];
    }
}



#define FLASH_BANK	CONFIG_FLASH_BANK


ulong flash_init(bd_t *bd)
{
    unsigned int    manufacture_ID = 0;
    unsigned int    device_ID = 0;



    // note Flash is in bank 0
    sram_config(FLASH_BANK, CFG_FLASH_BASE, 0, 0, SRAM_BANK_SIZE_8M, SRAM_BUS_WIDTH_16);

    sram_set_timing(FLASH_BANK, TIMING_PARAMETER);


    // disable memory bank 1
    sram_disable(1);

    Flash_Read_Silicon_ID(&manufacture_ID, &device_ID);
	
    if ((manufacture_ID & 0x0000FFFF) == 0x00C2)	/* MXIC's flash */
    {	
        if((device_ID & 0x0000FFFF) == 0x22C9)		/* MXIC_MX29LV640T */
        {            
            mxic_mx29lv640t_flash_init(bd);
        }        
	else if((device_ID & 0x0000FFFF) == 0x22CB)		/* MXIC_MX29LV640T */
        {          
		mxic_mx29lv640t_flash_init(bd);  
	}
        else
        {        
            printf("\nflash_init: flash type is not MXIC_MX29LV640T!!0x%x\n",(device_ID & 0x0000FFFF));
            
            return 0;
        }
    }

    return flash_info[0].size;
}



void flash_print_info(flash_info_t *info)
{
    int i;

    switch (info->flash_id & FLASH_VENDMASK)
    {
    	case (MXIC_MANUFACTURE_ID & FLASH_VENDMASK):
    	
            printf("MXIC: ");
            
            break;
        	
    	default:
    	
            printf("Unknown Vendor ");
            
            break;
    }

    switch (info->flash_id & FLASH_TYPEMASK)
    {
    	case (MXIC_DEVICE_ID_MX29LV640T & FLASH_TYPEMASK):
    	
            printf("1x MX29LV640T(64Mbit)\n");
            
            break;
		
    	default:
    	
            printf("Unknown Chip Type: %x\n", info->flash_id & FLASH_TYPEMASK);
            
            goto Done;
            
            break;
    }

    printf("  Size: %ld MB in %d Sectors\n", info->size >> 20, info->sector_count);

    printf("  Sector Start Addresses:");
    
    for (i = 0; i < info->sector_count; i++)
    {
        if ((i % 5) == 0)
        {
            printf ("\n   ");
        }
        
        printf (" %08lX%s", info->start[i], info->protect[i] ? " (RO)" : "     ");
    }
    
    printf ("\n");

Done:
}



int flash_error(ulong code)
{
    /* Check bit patterns */
    /* SR.7=0 is busy, SR.7=1 is ready */
    /* all other flags indicate error on 1 */
    /* SR.0 is undefined */
    /* Timeout is our faked flag */

    /* sequence is described in Intel 290644-005 document */

    /* check Timeout */
    if (code & BIT_TIMEOUT)
    {
		printf ("Timeout\n");
        return ERR_TIMOUT;
    }

    /* check Busy, SR.7 */
    if (~code & BIT_BUSY)
    {
        printf ("Busy\n");
        return ERR_PROG_ERROR;
    }

    /* check Vpp low, SR.3 */
    if (code & BIT_VPP_RANGE_ERROR)
    {
        printf ("Vpp range error\n");
        return ERR_PROG_ERROR;
    }

    /* check Device Protect Error, SR.1 */
    if (code & BIT_PROTECT_ERROR)
    {
        printf ("Device protect error\n");
        return ERR_PROG_ERROR;
    }

    /* check Command Seq Error, SR.4 & SR.5 */
    if (code & BIT_SEQUENCE_ERROR)
    {
        printf ("Command seqence error\n");
        return ERR_PROG_ERROR;
    }

    /* check Block Erase Error, SR.5 */
    if (code & BIT_ERASE_ERROR)
    {
        printf ("Block erase error\n");
        return ERR_PROG_ERROR;
    }

    /* check Program Error, SR.4 */
    if (code & BIT_PROGRAM_ERROR)
    {
        printf ("Program error\n");
        return ERR_PROG_ERROR;
    }

    /* check Block Erase Suspended, SR.6 */
    if (code & BIT_ERASE_SUSPEND)
    {
        printf ("Block erase suspended\n");
        return ERR_PROG_ERROR;
    }

    /* check Program Suspended, SR.2 */
    if (code & BIT_PROGRAM_SUSPEND)
    {
        printf ("Program suspended\n");
        return ERR_PROG_ERROR;
    }

    /* OK, no error */
    return ERR_OK;
}



/*
 * erase the sectors between s_first and s_last
 */
int flash_erase(flash_info_t *info, int s_first, int s_last)
{
    int iflag, cflag, prot, sect;
    int rc = ERR_OK;


    /* 
     * first look for protection bits
     */
    if (info->flash_id == FLASH_UNKNOWN)
    {
        return ERR_UNKNOWN_FLASH_TYPE;
    }

    if ((s_first < 0) || (s_first > s_last))
    {
        return ERR_INVAL;
    }

    if ((info->flash_id & FLASH_VENDMASK) != (MXIC_MANUFACTURE_ID & FLASH_VENDMASK)) 
    {
        return ERR_UNKNOWN_FLASH_VENDOR;
    }

    prot = 0;
    
    for (sect = s_first; sect <= s_last; ++sect) 
    {
        if (info->protect[sect])
        {
            prot++;
        }
    }
    
    if (prot)
    {
        return ERR_PROTECTED;
    }

    /*
     * Disable interrupts which might cause a timeout
     * here. Remember that our exception vectors are
     * at address 0 in the flash, and we don't want a
     * (ticker) exception to happen while the flash
     * chip is in programming mode.
     */
    cflag = icache_status();
    
    icache_disable();

    iflag = disable_interrupts();
   
    
    /* 
     * Start erase on unprotected sectors 
     */
    for (sect = s_first; sect <= s_last && !ctrlc(); sect++)
    {
    	int start_addr;
    	
    	int end_addr;
    	
    	printf("Erasing sector %2d ... ", sect);

    	if((info->flash_id & FLASH_VENDMASK) == (MXIC_MANUFACTURE_ID & FLASH_VENDMASK))
    	{
 	    //fLib_Flash_SectorErase(MX_Type, info->start[0], FLASH_FOUR, (info->start[sect] - info->start[0])/4);
 	    
 	    Flash_Sectors_Erase(sect, sect);
    	}    	
	else
	{
            printf("unknown flash type\n");
            
            return ERR_UNKNOWN_FLASH_TYPE;
	}
 
    	reset_timer_masked();
    	
    	start_addr = info->start[sect];
    	
        if (sect < flash_info[0].sector_count - 1)
        {
            end_addr = info->start[sect + 1];
        }
        else
        {
            end_addr = info->start[0] + info->size;
        }
        
        for (; start_addr < end_addr; start_addr += 2)
        {
            for (; *(volatile u16 *)start_addr != 0xFFFF; )
            {
                if (get_timer_masked() > CFG_FLASH_ERASE_TOUT)
                {
                    printf("fail:0x%x\n", start_addr);
                    
                    return ERR_PROG_ERROR;
                }
            }
    	}
    		
    	printf("ok.\n");
    }

    if (ctrlc())
    {
        printf("User Interrupt!\n");
    }

outahere:

    /* 
     * allow flash to settle - wait 10 ms 
     */
    udelay_masked(10000);

    if (iflag)
    {
        enable_interrupts();
    }

    if (cflag)
    {
        icache_enable();
    }

    return rc;
}


/*
 * Copy data from memory to flash
 */
volatile static int write_word(flash_info_t *info, ulong dest, ulong data)
{
    volatile ulong    new_data;
    volatile u16      new_data_16;
    int               flash_type;


    if ((info->flash_id & FLASH_VENDMASK) == (MXIC_MANUFACTURE_ID & FLASH_VENDMASK))
    {
        flash_type = MX_Type;
    }	
    else
    {
        printf("unknown flash type\n");
        
        return ERR_UNKNOWN_FLASH_TYPE;
    }


    Flash_Word_Write(dest, (u16)(data & 0x0000FFFF));
    
    Flash_Word_Write(dest + 2, (u16)((data >> 16) & 0x0000FFFF));    

    
    //fLib_Flash_ProgramWord(flash_type, info->start[0], dest, data);	
    //fLib_Flash_ReadWord(dest, &new_data);
	

    Flash_Word_Read(dest, &new_data_16);
    
    new_data = new_data_16 & 0x0000FFFF;
    
    Flash_Word_Read(dest + 2, &new_data_16);
    
    new_data |= (new_data_16 << 16) & 0xFFFF0000;

    if (new_data != data)
    {
        printf("addr: %x write error: should %x ==> result %x\n", dest, data, new_data);
        
        return ERR_PROG_ERROR;
    } 
	
    return ERR_OK;
}


/*
 * Copy memory to flash.
 *
 * src  : source address
 * addr : destination address
 *
 * return :
 *	ERR_OK : no error
 *	others : error code
 */
int write_buff(flash_info_t *info, uchar *src, ulong addr, ulong cnt)
{
    ulong    cp, wp, data;
    int      l;
    int      i, rc;


    /* get lower word aligned address */
    wp = (addr & ~3);

    /*
     * handle unaligned start bytes
     */
    if ((l = addr - wp) != 0) 
    {
        data = 0;
        
        for (i = 0, cp = wp; i < l; ++i, ++cp) 
        {
            data = (data >> 8) | (*(uchar *)cp << 24);
        }
        
        for (; i < 4 && cnt > 0; ++i) 
        {
            data = (data >> 8) | (*src++ << 24);
            
            --cnt;
            ++cp;
        }
        
        for (; cnt == 0 && i < 4; ++i, ++cp)
        {
            data = (data >> 8) | (*(uchar *)cp << 24);
        }

        if ((rc = write_word(info, wp, data)) != 0)
        {
            return (rc);
        }
        
        wp += 4;
    }

    /*
     * handle word aligned part
     */
    while (cnt >= 4) 
    {
        data = *((vulong *)src);
        
        if ((rc = write_word(info, wp, data)) != 0)
        {
            return (rc);
        }
        
        src += 4;
        wp  += 4;
        cnt -= 4;

	if((cnt & 0x3ff) == 0x0)
	{
            printf("\r0x%x       ",wp);
        }
    }
    
    printf("\n");

    if (cnt == 0) 
    {
        return ERR_OK;
    }

    /*
     * handle unaligned tail bytes
     */
    data = 0;
    
    for (i = 0, cp = wp; i < 4 && cnt > 0; ++i, ++cp) 
    {
        data = (data >> 8) | (*src++ << 24);
        
        --cnt;
    }
    
    for (; i < 4; ++i, ++cp) 
    {
        data = (data >> 8) | (*(uchar *)cp << 24);
    }

    return write_word(info, wp, data);
}
