/******************************************************************************
 *        
 * MODULE NAME:    star_gsw.c
 * PROJECT CODE:   Orion
 * DESCRIPTION:    
 * MAINTAINER:     MJLIU
 * DATE:           11 July 2003
 *
 * SOURCE CONTROL: 
 *
 * LICENSE:
 *     This source code is copyright (c) 2003 Star Semi Inc.
 *     All rights reserved.
 *
 * REVISION HISTORY:
 *    11 July 2003  -  MJLIU	- Initial Version v1.0
 *
 ******************************************************************************/

#include "str9100/star_gsw.h"



/******************************************************************************
 *
 * FUNCTION:  Hal_Gsw_Reset
 * PURPOSE:   
 *
 ******************************************************************************/
void Hal_Gsw_Reset(void)
{

}


/******************************************************************************
 *
 * FUNCTION:  Hal_Gsw_Initialize
 * PURPOSE:   
 *
 ******************************************************************************/
u_int32 Hal_Gsw_Initialize(GSW_OBJECT_T *pGsw_Object)
{
    u_int32              status = 0;
    u_int32              ii;
    ARL_TABLE_ENTRY_T    arl_table_entry;
    u_int32              delayed_interrrupt_config;
    

    /*
     * Write some default static ARL table entries : my VLAN MAC entries
     */
    for (ii = 0; ii < GSW_MAX_VLAN_NUM; ii++)
    {
        /*
         * Bypass this VLAN entry because of its invalid MAC address
         */
        if (!pGsw_Object->vlan[ii].vlan_mac[0] && 
            !pGsw_Object->vlan[ii].vlan_mac[1] &&
            !pGsw_Object->vlan[ii].vlan_mac[2] &&
            !pGsw_Object->vlan[ii].vlan_mac[3] &&
            !pGsw_Object->vlan[ii].vlan_mac[4] &&
            !pGsw_Object->vlan[ii].vlan_mac[5])
        {
            continue;
        }

        arl_table_entry.filter = 0;
    
        arl_table_entry.vlan_mac = 1;       // my VLAN MAC
    
        arl_table_entry.vlan_gid = pGsw_Object->vlan[ii].vlan_gid;
    
        arl_table_entry.age_field = 0x7;    // static entry
    
        arl_table_entry.port_map = pGsw_Object->vlan[ii].vlan_group;
    
        memcpy(arl_table_entry.mac_addr, pGsw_Object->vlan[ii].vlan_mac, 6);

        if (Hal_Gsw_Write_ARL_Table_Entry(&arl_table_entry))
        {
            status = 1;
            
           return status;
        }        
    }


    /*
     * Set Port-Based VLAN configuration
     */
    HAL_GSW_SET_MAC_PORT0_PVID(pGsw_Object->mac_port0_pvid);
    
    HAL_GSW_SET_MAC_PORT1_PVID(pGsw_Object->mac_port1_pvid);
    
    HAL_GSW_SET_CPU_PORT_PVID(pGsw_Object->cpu_port_pvid);


    /*
     * Set VLAN configuration
     */
    HAL_GSW_SET_VLAN_0_VID(pGsw_Object->vlan[0].vlan_vid);
    
    HAL_GSW_SET_VLAN_1_VID(pGsw_Object->vlan[1].vlan_vid);
    
    HAL_GSW_SET_VLAN_2_VID(pGsw_Object->vlan[2].vlan_vid);
    
    HAL_GSW_SET_VLAN_3_VID(pGsw_Object->vlan[3].vlan_vid);
    
    HAL_GSW_SET_VLAN_4_VID(pGsw_Object->vlan[4].vlan_vid);
    
    HAL_GSW_SET_VLAN_5_VID(pGsw_Object->vlan[5].vlan_vid);
    
    HAL_GSW_SET_VLAN_6_VID(pGsw_Object->vlan[6].vlan_vid);
    
    HAL_GSW_SET_VLAN_7_VID(pGsw_Object->vlan[7].vlan_vid);
    
    HAL_GSW_SET_VLAN_0_MEMBER(pGsw_Object->vlan[0].vlan_group);
    
    HAL_GSW_SET_VLAN_1_MEMBER(pGsw_Object->vlan[1].vlan_group);
    
    HAL_GSW_SET_VLAN_2_MEMBER(pGsw_Object->vlan[2].vlan_group);
    
    HAL_GSW_SET_VLAN_3_MEMBER(pGsw_Object->vlan[3].vlan_group);
    
    HAL_GSW_SET_VLAN_4_MEMBER(pGsw_Object->vlan[4].vlan_group);
    
    HAL_GSW_SET_VLAN_5_MEMBER(pGsw_Object->vlan[5].vlan_group);
    
    HAL_GSW_SET_VLAN_6_MEMBER(pGsw_Object->vlan[6].vlan_group);
    
    HAL_GSW_SET_VLAN_7_MEMBER(pGsw_Object->vlan[7].vlan_group);    
    
    HAL_GSW_SET_VLAN_0_TAG(pGsw_Object->vlan[0].vlan_tag_flag);
    
    HAL_GSW_SET_VLAN_1_TAG(pGsw_Object->vlan[1].vlan_tag_flag);
    
    HAL_GSW_SET_VLAN_2_TAG(pGsw_Object->vlan[2].vlan_tag_flag);
    
    HAL_GSW_SET_VLAN_3_TAG(pGsw_Object->vlan[3].vlan_tag_flag);
    
    HAL_GSW_SET_VLAN_4_TAG(pGsw_Object->vlan[4].vlan_tag_flag);
    
    HAL_GSW_SET_VLAN_5_TAG(pGsw_Object->vlan[5].vlan_tag_flag);
    
    HAL_GSW_SET_VLAN_6_TAG(pGsw_Object->vlan[6].vlan_tag_flag);
    
    HAL_GSW_SET_VLAN_7_TAG(pGsw_Object->vlan[7].vlan_tag_flag);


    /*
     * Set DMA configuration
     */
    HAL_GSW_TS_DMA_STOP();
    
    HAL_GSW_FS_DMA_STOP();
    
    HAL_GSW_WRITE_TSSD(pGsw_Object->dma.tssd);
    
    HAL_GSW_WRITE_TS_BASE(pGsw_Object->dma.ts_base);
    
    HAL_GSW_WRITE_FSSD(pGsw_Object->dma.fssd);
    
    HAL_GSW_WRITE_FS_BASE(pGsw_Object->dma.fs_base);


    if (pGsw_Object->dma.delayed_interrupt_use_flag)
    {
        delayed_interrrupt_config = (0x1 << 16) |
                                    ((pGsw_Object->dma.max_pending_fsrc_int_num & 0xFF) << 8) |
                                    (pGsw_Object->dma.max_pending_time & 0xFF);
                                    
        HAL_GSW_WRITE_DELAYED_INTERRUPT_CONFIG(delayed_interrrupt_config);
    }
    else
    {
        delayed_interrrupt_config = 0;
        
        HAL_GSW_WRITE_DELAYED_INTERRUPT_CONFIG(delayed_interrrupt_config);
    }

    /*
     * Register GSW's interrupt ISRs
     */

    return status;
}




/******************************************************************************
 *
 * FUNCTION:  Hal_Gsw_Write_PHY
 * PURPOSE:   
 *
 ******************************************************************************/
u_int32 Hal_Gsw_Write_PHY(u_int32 phy_addr, u_int32 phy_register, u_int16 write_data)
{
    u_int32 volatile    ii;
    
    
    /* 
     * clear previous rw_ok status
     */
    GSW_PHY_CONTROL = (0x1 << 15);
    
    GSW_PHY_CONTROL = ((phy_addr & 0x1) | 
                       ((phy_register & 0x1F) << 8) |
                       (0x1 << 13) |
                       ((write_data & 0xFFFF) << 16));

    for (ii = 0; ii < 0x100000; ii++)
    {       
        if ((GSW_PHY_CONTROL) & (0x1 << 15))
        {
            // clear the rw_ok status, and clear other bits value
            GSW_PHY_CONTROL = (0x1 << 15);

            return (1);
        }
    }

    return (0);
}


/******************************************************************************
 *
 * FUNCTION:  Hal_Gsw_Read_PHY
 * PURPOSE:   
 *
 ******************************************************************************/
u_int32 Hal_Gsw_Read_PHY(u_int32 phy_addr, u_int32 phy_register, u_int16 *read_data)
{
    u_int32 volatile    status;
    u_int32 volatile    ii;
    
    
    /* 
     * clear previous rw_ok status
     */
    GSW_PHY_CONTROL = (0x1 << 15);
    
    GSW_PHY_CONTROL = ((phy_addr & 0x1) | 
                       ((phy_register & 0x1F) << 8) | 
                       (0x1 << 14));    

    for (ii = 0; ii < 0x100000; ii++)
    {
    	status = GSW_PHY_CONTROL;
        
        if (status & (0x1 << 15))
        {
            // clear the rw_ok status, and clear other bits value
            GSW_PHY_CONTROL = (0x1 << 15);
        	
            *read_data = (u_int16)((status >> 16) & 0xFFFF);
            
            return (1);
        }
    }

    return (0);
}



/******************************************************************************
 *
 * FUNCTION:  Hal_Gsw_Write_IP_TOS_Priority
 * PURPOSE:   
 *
 ******************************************************************************/
void Hal_Gsw_Write_IP_TOS_Priority(u_int32 ip_tos, u_int32 priority_value)
{
    u_int32             mod_value = 0;
    u_int32 volatile    temp_value;
    
    
    mod_value = ip_tos % 8;
    
    if (ip_tos <= 7)
    {
        temp_value = GSW_IP_TOS_0_7_PRIORITY;
        
        temp_value &= ~(0x7 << mod_value);
        
        temp_value |= ((priority_value & 0x7) << mod_value);
        
        GSW_IP_TOS_0_7_PRIORITY = temp_value;
    }
    else if ((ip_tos >= 8) && (ip_tos <= 15))
    {
        temp_value = GSW_IP_TOS_8_15_PRIORITY;
        
        temp_value &= ~(0x7 << mod_value);
        
        temp_value |= ((priority_value & 0x7) << mod_value);
        
        GSW_IP_TOS_8_15_PRIORITY = temp_value;
    }
    else if ((ip_tos >= 16) && (ip_tos <= 23))
    {
        temp_value = GSW_IP_TOS_16_23_PRIORITY;
        
        temp_value &= ~(0x7 << mod_value);
        
        temp_value |= ((priority_value & 0x7) << mod_value);
        
        GSW_IP_TOS_16_23_PRIORITY = temp_value;
    }
    else if ((ip_tos >= 24) && (ip_tos <= 31))
    {
        temp_value = GSW_IP_TOS_24_31_PRIORITY;
        
        temp_value &= ~(0x7 << mod_value);
        
        temp_value |= ((priority_value & 0x7) << mod_value);
        
        GSW_IP_TOS_24_31_PRIORITY = temp_value;
    }
    else if ((ip_tos >= 32) && (ip_tos <= 39))
    {
        temp_value = GSW_IP_TOS_32_39_PRIORITY;
        
        temp_value &= ~(0x7 << mod_value);
        
        temp_value |= ((priority_value & 0x7) << mod_value);
        
        GSW_IP_TOS_32_39_PRIORITY = temp_value;
    }
    else if ((ip_tos >= 40) && (ip_tos <= 47))
    {
        temp_value = GSW_IP_TOS_40_47_PRIORITY;
        
        temp_value &= ~(0x7 << mod_value);
        
        temp_value |= ((priority_value & 0x7) << mod_value);
        
        GSW_IP_TOS_40_47_PRIORITY = temp_value;
    }
    else if ((ip_tos >= 48) && (ip_tos <= 55))
    {
        temp_value = GSW_IP_TOS_48_55_PRIORITY;
        
        temp_value &= ~(0x7 << mod_value);
        
        temp_value |= ((priority_value & 0x7) << mod_value);
        
        GSW_IP_TOS_48_55_PRIORITY = temp_value;
    }
    else if ((ip_tos >= 56) && (ip_tos <= 63))
    {
        temp_value = GSW_IP_TOS_56_63_PRIORITY;
        
        temp_value &= ~(0x7 << mod_value);
        
        temp_value |= ((priority_value & 0x7) << mod_value);
        
        GSW_IP_TOS_56_63_PRIORITY = temp_value;
    }
}


/******************************************************************************
 *
 * FUNCTION:  Hal_Gsw_Search_ARL_Table_Entry
 * PURPOSE:   
 *
 ******************************************************************************/
int Hal_Gsw_Search_ARL_Table_Entry(u_int32 command_type)
{
    u_int32 volatile    ii;
    u_int32 volatile    search_result;
    
    
    GSW_ARL_TABLE_ACCESS_CONTROL_0 = 0x0;
    
    GSW_ARL_TABLE_ACCESS_CONTROL_1 = 0x0;
    
    GSW_ARL_TABLE_ACCESS_CONTROL_2 = 0x0;

    
    if ((command_type == 1) || (command_type == 2))
    {
        // issue the search_start or search_again command
        GSW_ARL_TABLE_ACCESS_CONTROL_0 = command_type;
    }    
    else
    {
        // invalid command; regard as timeout
        return (-1);
    }
    
    for (ii = 0; ii < 0x100000; ii++)
    {
    	search_result = GSW_ARL_TABLE_ACCESS_CONTROL_1;
    	
        if (search_result & (0x1))  // search command is complete
        {
            break;
        }
    }
    
    if (ii < 0x100000)
    {
        if (search_result & (0x1 << 2))  // find one valid entry
        {
            if (search_result & (0x1 << 1))  // reach the end of ARL table
            {
                // one valid ARL entry is found, and the end of ARL table is reached
                return (2);
            }
            else
            {
                // one valid ARL entry is found, and the end of ARL table is not reached
                return (1);
            }
        }
        else
        {
            /* 
             * Since no valid ARL entry is found, it is reasonable to believe the end of
             * ARL table is reached
             */
            return (0);
        }                          
    }
    else  // timeout, something wrong
    {
        return (-1);
    }
}



/******************************************************************************
 *
 * FUNCTION:  Hal_Gsw_Find_ARL_Table_Valid_Entry_Num
 * PURPOSE:   
 *
 ******************************************************************************/
int Hal_Gsw_Find_ARL_Table_Valid_Entry_Num(void)
{
    int        search_result = 0;
    u_int32    entry_num = 0;
    
    
    /*
     * issue search_start command
     */
    search_result = Hal_Gsw_Search_ARL_Table_Entry(1);
    
    if (search_result < 0)
    {
        // search command timeout
        return (-1);
    }
    else if (search_result == 0)
    {
        // no valid ARL entry found
        return (0);
    }
    else if (search_result == 2)
    {
        // one valid ARL entry found, and the end of ARL table is reached
        return (1);
    }
    else if (search_result == 1)
    {     
        entry_num++;        
        
        while (1)
        {
            /*
             * issue search_again command
             */
            search_result = Hal_Gsw_Search_ARL_Table_Entry(2);       
        
            if (search_result < 0)
            {
                // command timeout
                return (-1);
            }
            else if (search_result == 0)
            {
                // no more valid ARL entry found again
                break;
            }
            else if (search_result == 2)
            {
                // one valid ARL entry found, and the end of ARL table is reached
                entry_num++;
                
                break;
            }
            else if (search_result == 1)
            {
                // one valid ARL entry found, and the end of ARL table is not reached
                entry_num++;
            }
            else
            {
                // something wrong
                return (-1);
            }
        }  // end of while (1)
        
        return (entry_num);
    }
    else
    {
        // something wrong
        return (-1);
    }
}



/******************************************************************************
 *
 * FUNCTION:  Hal_Gsw_Write_ARL_Table_Entry
 * PURPOSE:   
 *
 ******************************************************************************/
u_int32 Hal_Gsw_Write_ARL_Table_Entry(ARL_TABLE_ENTRY_T *arl_table_entry)
{
    u_int32 volatile    ii;
    
    
    GSW_ARL_TABLE_ACCESS_CONTROL_0 = 0x0;
    
    GSW_ARL_TABLE_ACCESS_CONTROL_1 = 0x0;
    
    GSW_ARL_TABLE_ACCESS_CONTROL_2 = 0x0;
    
    GSW_ARL_TABLE_ACCESS_CONTROL_1 = (((arl_table_entry->filter & 0x1) << 3) | 
                                      ((arl_table_entry->vlan_mac & 0x1) << 4) | 
                                      ((arl_table_entry->vlan_gid & 0x7) << 5) |
                                      ((arl_table_entry->age_field & 0x7) << 8) |
                                      ((arl_table_entry->port_map & 0x7) << 11) |
                                      ((arl_table_entry->mac_addr[0] & 0xFF) << 16) |
                                      ((arl_table_entry->mac_addr[1] & 0xFF) << 24));
    
    GSW_ARL_TABLE_ACCESS_CONTROL_2 = (((arl_table_entry->mac_addr[2] & 0xFF) << 0) |
                                      ((arl_table_entry->mac_addr[3] & 0xFF) << 8) |
                                      ((arl_table_entry->mac_addr[4] & 0xFF) << 16) |
                                      ((arl_table_entry->mac_addr[5] & 0xFF) << 24));

    /*
     * issue the write command
     */
    GSW_ARL_TABLE_ACCESS_CONTROL_0 = (0x1 << 3);
    
    for (ii = 0; ii < 0x100000; ii++)
    {
        if (GSW_ARL_TABLE_ACCESS_CONTROL_1 & (0x1))
        {
            return (0);  // write OK
        }
    }
    
    return (1);  // write failed
}


/******************************************************************************
 *
 * FUNCTION:  Hal_Gsw_Lookup_ARL_Table_Entry
 * PURPOSE:   
 *
 ******************************************************************************/
u_int32 Hal_Gsw_Lookup_ARL_Table_Entry(ARL_TABLE_ENTRY_T *arl_table_entry)
{
    u_int32 volatile    ii;
    u_int32 volatile    lookup_result = 0;


    GSW_ARL_TABLE_ACCESS_CONTROL_0 = 0x0;
    
    GSW_ARL_TABLE_ACCESS_CONTROL_1 = 0x0;
    
    GSW_ARL_TABLE_ACCESS_CONTROL_2 = 0x0;
    
    GSW_ARL_TABLE_ACCESS_CONTROL_1 = (((arl_table_entry->vlan_gid & 0x7) << 5) |
                                      ((arl_table_entry->mac_addr[0] & 0xFF) << 16) |
                                      ((arl_table_entry->mac_addr[1] & 0xFF) << 24));
    
    GSW_ARL_TABLE_ACCESS_CONTROL_2 = (((arl_table_entry->mac_addr[2] & 0xFF) << 0) |
                                      ((arl_table_entry->mac_addr[3] & 0xFF) << 8) |
                                      ((arl_table_entry->mac_addr[4] & 0xFF) << 16) |
                                      ((arl_table_entry->mac_addr[5] & 0xFF) << 24));

    // issue the Lookup command
    GSW_ARL_TABLE_ACCESS_CONTROL_0 |= (0x1 << 2);
    
    for (ii = 0; ii < 0x100000; ii++)
    {
        if (GSW_ARL_TABLE_ACCESS_CONTROL_1 & (0x1))  // check command complete or not
        {
            break;	
        }
    }

    if (ii < 0x100000)
    {
        lookup_result = GSW_ARL_TABLE_ACCESS_CONTROL_1;
        
        if (lookup_result & (0x1 << 2))
        {
            arl_table_entry->filter = ((lookup_result >> 3) & 0x1);
        
            arl_table_entry->vlan_mac = ((lookup_result >> 4) & 0x1);
        
            arl_table_entry->age_field = ((lookup_result >> 8) & 0x7);
        
            arl_table_entry->port_map = ((lookup_result >> 11) & 0x7);
        
            return (1);  // matched ARL entry is found
        }
    }
    
    return (0);  // no matched ARL entry is found
}



