/******************************************************************************
 *        
 * MODULE NAME:    star_led.c
 * PROJECT CODE:   Orion
 * DESCRIPTION:    
 * MAINTAINER:     MJLIU
 * DATE:           30 June 2003
 *
 * SOURCE CONTROL: 
 *
 * LICENSE:
 *     This source code is copyright (c) 2003 Star Semi Inc.
 *     All rights reserved.
 *
 * REVISION HISTORY:
 *    30 June 2003  -  MJLIU	- Initial Version v1.0
 *
 ******************************************************************************/
 
#include "str9100/star_led.h"

u_int32    gpio_valid_mask = 0;


#define LED_DELAY_COUNTER    0x1000


/******************************************************************************
 *
 * FUNCTION:  Hal_Led_Initialize
 * PURPOSE:   
 *
 ******************************************************************************/
void Hal_Led_Initialize(void)
{
    u_int32 volatile    reset_latch_value;
    u_int32 volatile    ii, jj;


    reset_latch_value = *(u_int32 volatile *)(0x77000014);
    

    if (!(reset_latch_value & (0x1 << 4)))
    {
        /*
         * treat Multi-ICE pins as GPIO pins
         */
        gpio_valid_mask |= (0x1 << 0) | (0x1 << 1) | (0x1 << 2) | (0x1 << 3) |
                           (0x1 << 18) | (0x1 << 19) | (0x1 << 20);
    }

    if (!(reset_latch_value & (0x1 << 9)))
    {
        /*
         * treat UART pins as GPIO pins
         */
        gpio_valid_mask |= (0x1 << 4) | (0x1 << 5) | (0x1 << 6) | (0x1 << 7);
    }

    if (reset_latch_value & (0x1 << 14))
    {
        /*
         * treat MAC port 0's CRS and COL pins as GPIO pins
         */
        gpio_valid_mask |= (0x1 << 8) | (0x1 << 9);
    }

    if (reset_latch_value & (0x1 << 15))
    {
        /*
         * treat MAC port 1's CRS and COL pins as GPIO pins
         */
        gpio_valid_mask |= (0x1 << 10) | (0x1 << 11);
    }

    if (reset_latch_value & (0x1 << 16))
    {
        /*
         * treat PCMCIA pins as GPIO pins
         */
        gpio_valid_mask |= (0x1 << 12) | (0x1 << 13) | (0x1 << 14) | (0x1 << 15) |
                           (0x1 << 16) | (0x1 << 17);
    }

	
    /*
     * Configure all GPIO pins as follows:
     * 1. output pins
     * 2. disable pin bypass mode
     * 3. turn all leds off
     * 4. sequentially turn all leds on and all leds off     
     *    then, we can set GPIO Low to turn LED On, and GPIO High to turn LED Off 
     */
    if (gpio_valid_mask)
    {
        HAL_GPIO_SET_DIRECTION_OUTPUT(gpio_valid_mask);
    
        HAL_GPIO_DISABLE_BYPASS_MODE(gpio_valid_mask);

        Hal_Led_All_Off();
    
        for (ii = 0; ii < MAX_LED_NUMBER; ii++)
        {
            Hal_Led_On(LED_0_MASK << ii);
        
            for (jj = 0; jj < LED_DELAY_COUNTER; jj++);
        
            Hal_Led_Off(LED_0_MASK << ii);
        
            for (jj = 0; jj < LED_DELAY_COUNTER; jj++);
        }
    }
}


/******************************************************************************
 *
 * FUNCTION:  Hal_Led_All_On
 * PURPOSE:   
 *
 ******************************************************************************/
void Hal_Led_All_On(void)
{
    /*
     * perform Write Low to GPIO Pin
     */
    if (gpio_valid_mask)
    {
        HAL_GPIO_SET_DATA_OUT_LOW(gpio_valid_mask);
    }
}


/******************************************************************************
 *
 * FUNCTION:  Hal_Led_All_Off
 * PURPOSE:   
 *
 ******************************************************************************/
void Hal_Led_All_Off(void)
{
    /*
     * perform Write High to GPIO Pin
     */
    if (gpio_valid_mask)
    {
        HAL_GPIO_SET_DATA_OUT_HIGH(gpio_valid_mask);
    }
}


/******************************************************************************
 *
 * FUNCTION:  Hal_Led_On
 * PURPOSE:   
 *
 ******************************************************************************/
void Hal_Led_On(u_int32 led_index)
{
    /*
     * perform Write Low to GPIO Pin
     */
    if (led_index & gpio_valid_mask)
    {
        HAL_GPIO_SET_DATA_OUT_LOW(led_index & gpio_valid_mask);
    }
}


/******************************************************************************
 *
 * FUNCTION:  Hal_Led_Off
 * PURPOSE:   
 *
 ******************************************************************************/
void Hal_Led_Off(u_int32 led_index)
{
    /*
     * perform Write High to GPIO Pin
     */
    if (led_index & gpio_valid_mask)
    {
        HAL_GPIO_SET_DATA_OUT_HIGH(led_index & gpio_valid_mask);
    }
}


/******************************************************************************
 *
 * FUNCTION:  Hal_Led_Toggle
 * PURPOSE:   
 *
 ******************************************************************************/
void Hal_Led_Toggle(u_int32 led_index)
{     
    u_int32  data_out_state = 0;


    /*
     * 1. read GPIO Data Out State
     * 2. if GPIO High, turn LED on, otherwise, turn LED off
     */
    HAL_GPIO_READ_DATA_OUT_STATE(data_out_state);
    
    if (data_out_state & led_index & gpio_valid_mask)
    {
        // GPIO High, i.e., LED is off. Now, turn it on
        Hal_Led_On(led_index & gpio_valid_mask);
    }
    else
    {
        // GPIO Low, i.e., LED is on. Now turn it off
        Hal_Led_Off(led_index & gpio_valid_mask);
    }
}