/******************************************************************************
 *        
 * MODULE NAME:    star_uart.c
 * PROJECT CODE:   Orion
 * DESCRIPTION:    
 * MAINTAINER:     MJLIU
 * DATE:           09 July 2003
 *
 * SOURCE CONTROL: 
 *
 * LICENSE:
 *     This source code is copyright (c) 2003 Star Semi Inc.
 *     All rights reserved.
 *
 * REVISION HISTORY:
 *    09 July 2003  -  MJLIU	- Initial Version v1.0
 *
 ******************************************************************************/
#include <stdio.h> 
#include "str9100/star_uart.h"


/******************************************************************************
 *
 * FUNCTION:  Uart_Configure_Baud_Rate
 * PURPOSE:   
 *
 ******************************************************************************/
void Uart_Configure_Baud_Rate(unsigned int baud_rate, unsigned int input_clock)
{
    unsigned int  baud_rate_divisor;
    unsigned int  updated_input_clock;

    /*
     * For 48MHz input clock and new UART Pre_Scalar design
     */
    input_clock = 48000000;
    updated_input_clock = (input_clock/325) * 100;
    	    	
    baud_rate_divisor = updated_input_clock/(16 * baud_rate);    	

    
    if (baud_rate_divisor)
    {    
        UART_WRITE_DLM((baud_rate_divisor & 0x0000FF00) >> 8);        
        UART_WRITE_DLL((baud_rate_divisor & 0x000000FF));
    }
}


/******************************************************************************
 *
 * FUNCTION:  Uart_Control_Line_Control
 * PURPOSE:   
 *
 ******************************************************************************/
void Uart_Configure_Line_Control(u_int32 baud_rate, u_int32 word_len, u_int32 parity, u_int32 stop_bit)
{
    u_int32  uart_LCR = 0;
    
    
    // configure baud rate setting
    Uart_Configure_Baud_Rate(baud_rate, UART_INPUT_CLOCK);
    
    // configure word length setting
    if (word_len == WORD_FIVE_BITS)
    {
        uart_LCR |= WORD_LENGTH_5;
    }
    else if (word_len == WORD_SIX_BITS)
    {
        uart_LCR |= WORD_LENGTH_6;
    }
    else if (word_len == WORD_SEVEN_BITS)
    {
        uart_LCR |= WORD_LENGTH_7;
    }
    else if (word_len == WORD_EIGHT_BITS)
    {
        uart_LCR |= WORD_LENGTH_8;
    }
    
    // configure parity setting
    if (parity == NONE_PARITY)
    {
        uart_LCR |= PARITY_CHECK_NONE;
    }
    else if (parity == EVEN_PARITY)
    {
        uart_LCR |= PARITY_CHECK_EVEN;
    }
    else if (parity == ODD_PARITY)
    {
        uart_LCR |= PARITY_CHECK_ODD;
    }
    else if (parity == ONE_PARITY)
    {
        uart_LCR |= PARITY_CHECK_STICK_ONE;
    }
    else if (parity == ZERO_PARITY)
    {
        uart_LCR |= PARITY_CHECK_STICK_ZERO;
    }

    // configure stop bit setting
    if (stop_bit == ONE_STOP_BIT)
    {
        uart_LCR |= STOP_BIT_1;
    }
    else if ((stop_bit == ONE_HALF_STOP_BIT) || (stop_bit == TWO_STOP_BIT))
    {
        uart_LCR |= STOP_BIT_2;
    }
    
    UART_LCR = uart_LCR;
}


/******************************************************************************
 *
 * FUNCTION:  Uart_Configure_FIFO
 * PURPOSE:   
 *
 ******************************************************************************/
void Uart_Configure_FIFO(u_int32 fifo_flag, u_int32 rx_fifo_trigger_level)
{
    u_int32  uart_FCR = 0;

    if (fifo_flag == TX_RX_FIFO_DISABLE)
    {
        uart_FCR = 0;
    }
    else
    {
        uart_FCR |= FIFO_ENABLE;
    	
        if (rx_fifo_trigger_level == 1)
        {
            uart_FCR |= RX_FIFO_TRIGGER_LEVEL_1;
        }
        else if (rx_fifo_trigger_level == 4)
        {
            uart_FCR |= RX_FIFO_TRIGGER_LEVEL_4;
        }
        else if (rx_fifo_trigger_level == 8)
        {
            uart_FCR |= RX_FIFO_TRIGGER_LEVEL_8;
        }
        else if (rx_fifo_trigger_level == 14)
        {
            uart_FCR |= RX_FIFO_TRIGGER_LEVEL_14;
        }
        
        // reset TX/RX FIFO
        uart_FCR |= (RX_FIFO_RESET | TX_FIFO_RESET);
    }
    
    UART_FCR = uart_FCR;
}

