/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 * Gary Jennejohn <gj@denx.de>
 *
 * Configuation settings for the SAMSUNG board.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#define CFG_MATRIX400
/*
 * If we are developing, we might want to start armboot from ram
 * so we MUST NOT initialize critical regs like mem-timing ...
 */
#define CONFIG_INIT_CRITICAL		/* undef for developing */
#undef CONFIG_BOOTBINFUNC

/*
 * High Level Configuration Options
 * (easy to change)
 */
#define CONFIG_STR9100_GSW
#define CONFIG_DM9161AE
#undef CONFIG_USE_IRQ			/* we don't need IRQ/FIQ stuff */

#define CONFIG_RTC_RS5C372A
#define CFG_I2C_RTC_ADDR 0x32
/*
 * define Data Bus Width of DDRC
 */
#define DDRC_DATA_BUS_WIDTH_16	(0)		/* the data bus width of DDRC */
#define DDRC_DATA_BUS_WIDTH_32	(1)
#define DDRC_DATA_BUS_WIDTH DDRC_DATA_BUS_WIDTH_16

/*
 * Note the DDR SDRAM module used on STR9100 hardware platform is x16 device
 */
#define DDRC_SDRAM_SIZE_64MBIT	(0)	/* the size of individual DDR SDRAM module */
#define DDRC_SDRAM_SIZE_128MBIT	(1)
#define DDRC_SDRAM_SIZE_256MBIT	(2)
#define DDRC_SDRAM_SIZE_512MBIT	(3)
#define DDRC_SDRAM_MODULE_SIZE	DDRC_SDRAM_SIZE_256MBIT

#define CONFIG_SOFT_I2C
#define CFG_I2C_SPEED	100
#define CFG_I2C_SLAVE	0
#define CFG_I2C_EEPROM_ADDR_LEN 1
#define CFG_EEPROM_AT24C16
#define	CFG_I2C_EEPROM_ADDR		0x50

#define CFG_ENV_IS_IN_EEPROM
#ifdef  CFG_ENV_IS_IN_EEPROM
#define CFG_ENV_OFFSET			1024 
#define CFG_ENV_SIZE			1024 
#define CFG_EEPROM_SIZE			2*1024
#endif

#define CFG_LOAD_ADDR		0x21000000  /* default load address */
/*
 * Hardware drivers
 */
#define CONFIG_STR9100_GSW

#define CONFIG_ORN_ETH1000


#ifdef not_complete_yet
#define FMAC_BASE		0x07000300	/* agrees with WIN CE PA */
#define CS8900_BUS16		1		/* the Linux driver does accesses as shorts */
#endif

/*
 * select serial console configuration
 */
#define CONFIG_SERIAL1          1		/* use COM1 as terminal */

/* allow to overwrite serial and ethaddr */
//#define CONFIG_ENV_OVERWRITE

#define CONFIG_BAUDRATE		115200 /*38400*/
#define CFG_TL16C752B

// define the commands provided by armboot like: go, run, bootm,...
//#define CONFIG_COMMANDS		(CONFIG_CMD_DFL & ~CFG_CMD_CACHE)
#define CONFIG_COMMANDS		\
		       ((CONFIG_CMD_DFL | \
			CFG_CMD_BDI  |\
			CFG_CMD_SDRAM |\
			CFG_CMD_IMI  |\
			CFG_CMD_DATE|\
			CFG_CMD_NET|\
			CFG_CMD_PING |\
			CFG_CMD_DHCP|\
			CFG_CMD_I2C |\
			CFG_CMD_EEPROM|\
			CFG_CMD_CACHE) & \
		      ~(CFG_CMD_AUTOSCRIPT | \
			CFG_CMD_MISC | \
			CFG_CMD_LOADS \
			))


/* this must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>

#define	CONFIG_BOOTDELAY	3

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CONFIG_KGDB_BAUDRATE	115200		/* speed to run kgdb serial port */
	
/* what's this ? it's not used anywhere */
#define CONFIG_KGDB_SER_INDEX	1		/* which serial port to use */
#endif

/*
 * Miscellaneous configurable options
 */
#define	CFG_LONGHELP				/* undef to save memory	*/
#define	CFG_PROMPT		"Artila>"	/* Monitor Command Prompt */
#define	CFG_CBSIZE		256		/* Console I/O Buffer Size */
#define	CFG_PBSIZE		(CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */
#define	CFG_MAXARGS		16		/* max number of command args	*/
#define CFG_BARGSIZE		CFG_CBSIZE	/* Boot Argument Buffer Size	*/


#define CFG_MEMTEST_START	0x20100000	/* memtest works on	*/
#define CFG_MEMTEST_END		0x20400000	/* 64 MB in DRAM	*/

#undef  CFG_CLKS_IN_HZ				/* everything, incl board info, in Hz */
#define CFG_HZ			1000		/* timer clock frequency */


/* valid baudrates */
#define CFG_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }


#ifndef __ASSEMBLY__
/*-----------------------------------------------------------------------
 * Board specific extension for bd_info
 *
 * This structure is embedded in the global bd_info (bd_t) structure
 * and can be used by the board specific code (eg board/...)
 */

struct bd_info_ext
{
    /* helper variable for board environment handling
     *
     * env_crc_valid == 0    =>   uninitialised
     * env_crc_valid  > 0    =>   environment crc in flash is valid
     * env_crc_valid  < 0    =>   environment crc in flash is invalid
     */
     int	env_crc_valid;
};
#endif


/*-----------------------------------------------------------------------
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CFG_FLASH_CFI 1
#define CFG_FLASH_CFI_DRIVER 1
#undef CFG_ENV_IS_IN_FLASH	
#ifdef CFG_ENV_IS_IN_FLASH
#define CFG_ENV_SECT_SIZE    0x10000 
#define CFG_ENV_SIZE		0x10000		/* Total Size of Environment Sector */
#endif

#define CONFIG_STACKSIZE	(128*1024)	/* regular stack */

#define CFG_MALLOC_LEN	(0x10000 + 128*1024)
#define CFG_GBL_DATA_SIZE	128	/* size in bytes reserved for initial data */
#ifdef CONFIG_USE_IRQ
#define CONFIG_STACKSIZE_IRQ	(4 * 1024)	/* IRQ stack */
#define CONFIG_STACKSIZE_FIQ	(4 * 1024)	/* FIQ stack */
#endif

/*-----------------------------------------------------------------------
 * Physical Memory Map
 */
#define CONFIG_NR_DRAM_BANKS	(1)		/* we have 1 bank of DRAM */
#define PHYS_SDRAM 0x20000000
#define PHYS_SDRAM_SIZE 0x2000000  /* 32 megs */

#define PHYS_FLASH_1		0x10000000	/* Flash Bank #1 */
#define CFG_FLASH_BASE		PHYS_FLASH_1


/*-----------------------------------------------------------------------
 * FLASH and environment organization
 */
#define CFG_MAX_FLASH_BANKS	(1)		/* max number of memory banks */
#define CFG_MAX_FLASH_SECT	(512)		/* max number of sectors on one chip */


/* timeout values are in ticks */
#define CFG_FLASH_ERASE_TOUT	(5*CFG_HZ)	/* Timeout for Flash Erase */
#define CFG_FLASH_WRITE_TOUT	(5*CFG_HZ)	/* Timeout for Flash Write */


#define CFG_ENV_ADDR		(PHYS_FLASH_1 + 0x20000)	/* Addr of Environment Sector */

#define CFG_CONSOLE_INFO_QUIET

#ifdef CONFIG_USE_IRQ
#error CONFIG_USE_IRQ not supported
#endif

#endif	/* __CONFIG_H */
